package main

import (
	"gitea.deepak.science/deepak/gogmagog/config"
	"gitea.deepak.science/deepak/gogmagog/models"
	"gitea.deepak.science/deepak/gogmagog/routes"
	"gitea.deepak.science/deepak/gogmagog/store"
	"gitea.deepak.science/deepak/gogmagog/tokens"
	"log"
	"net/http"
	"os"
)

func main() {
	// Config
	conf, err := config.GetConf("config")
	if err != nil {
		log.Println("Could not get config", err)
		os.Exit(1)
	}

	appConf := conf.App
	port := appConf.Port
	env := appConf.Environment

	// DB
	store, err := store.GetStore(&conf.Db)
	if err != nil {
		log.Println("Could not get store", err)
		os.Exit(1)

	}

	if store != nil {
		log.Println("Got DB connection")
	}
	m := models.New(store)
	if m != nil {
		log.Println("created model")
	}

	router := routes.NewRouter(m, tokens.New("my secret"))

	log.Println("Running server on " + port)

	http.ListenAndServe(":"+port, router)

	log.Println("App environment is " + env)
}
